"""
@file
    OBSW_Config.py
@date
    01.11.2019
@brief
    Global settings for UDP client
"""
import enum
from typing import Tuple

"""
Global service_type definitions
"""
ethernetAddressT = Tuple[str, int]


# Mode options, set by args parser
class ModeList(enum.Enum):
    GUIMode = 0
    ListenerMode = 1
    SingleCommandMode = 2
    ServiceTestMode = 3
    SoftwareTestMode = 4
    UnitTest = 5


class ComIF(enum.Enum):
    Ethernet = 0
    Serial = 1


"""
Mission/Device specific information.
"""

# TODO: Automate / Autofill this file with the MIB parser

# Object IDs
GPS0_ObjectId = bytearray([0x44, 0x10, 0x1F, 0x00])
GPS1_ObjectId = bytearray([0x44, 0x20, 0x20, 0x00])
DUMMY_DEVICE_ID = bytearray([0x44, 0x00, 0xAF, 0xFE])

# SIDs
GPS0_SID = bytearray([0x00, 0x00, 0x1f, 0x00])
GPS1_SID = bytearray([0x00, 0x00, 0x2f, 0x00])

"""
All global variables, set in main program with arg parser
"""


# General Settings
G_SCRIPT_MODE = 1
G_MODE_ID = 0
G_SERVICE = 17
G_DISPLAY_MODE = "long"


G_COM_IF = 0
# COM Port for serial communication
G_COM_PORT = 'COM0'
G_SERIAL_TIMEOUT = 0.01
# Time related
G_TM_TIMEOUT = 10
G_TC_SEND_TIMEOUT_FACTOR = 2.0

# Ethernet connection settings
G_REC_ADDRESS = 0
G_SEND_ADDRESS = (0, 0)

# Print Settings
G_PRINT_TO_FILE = True
G_PRINT_HK_DATA = False
G_PRINT_RAW_TM = False

"""
These objects are set for the Unit Test, no better solution found yet
"""
G_TM_LISTENER = None
G_COM_INTERFACE = None
G_TMTC_PRINTER = None


# noinspection PyUnusedLocal
def setGlobals(args):
    global G_REC_ADDRESS, G_SEND_ADDRESS, G_SCRIPT_MODE, G_MODE_ID, G_SERVICE, G_DISPLAY_MODE,\
        G_COM_IF, G_COM_PORT, G_SERIAL_TIMEOUT, G_TM_TIMEOUT, G_TC_SEND_TIMEOUT_FACTOR, \
        G_PRINT_TO_FILE, G_PRINT_HK_DATA, G_PRINT_RAW_TM
    if args.mode == 0:
        print("GUI mode not implemented yet !")
    if args.shortDisplayMode:
        G_DISPLAY_MODE = "short"
    else:
        G_DISPLAY_MODE = "long"
    # Board IP address and ethernet port IP address can be passed optionally
    # by passing line parameter In PyCharm: Set parameters in run configuration
    # Add IP address of Ethernet port. Use command ipconfig in windows console or ifconfig in Linux.
    portReceive = 2008
    recAddressToSet = (args.clientIP, portReceive)
    # Static IP of board
    portSend = 7
    sendAddressToSet = (args.boardIP, portSend)
    if 0 <= args.mode <= 5:
        if args.mode == 0:
            G_MODE_ID = ModeList.GUIMode
        elif args.mode == 1:
            G_MODE_ID = ModeList.ListenerMode
        elif args.mode == 2:
            G_MODE_ID = ModeList.SingleCommandMode
        elif args.mode == 3:
            G_MODE_ID = ModeList.ServiceTestMode
        elif args.mode == 4:
            G_MODE_ID = ModeList.SoftwareTestMode
        elif args.mode == 5:
            G_MODE_ID = ModeList.UnitTest
    else:
        print("Invalid mode argument, setting default mode (1)")
        G_MODE_ID = ModeList[1]

    if args.comIF == 0:
        G_COM_IF = ComIF.Ethernet
    elif args.comIF == 1:
        G_COM_IF = ComIF.Serial
    else:
        print("Invalid Communication Interface, setting serial (1)")
        G_COM_IF = ComIF.Serial

    G_SERVICE = str(args.service)
    if G_SERVICE.isdigit():
        G_SERVICE = int(args.service)
    else:
        G_SERVICE = args.service
    G_REC_ADDRESS = recAddressToSet
    G_SEND_ADDRESS = sendAddressToSet
    G_MODE_ID = G_MODE_ID
    G_PRINT_HK_DATA = args.print_hk
    G_PRINT_TO_FILE = args.print_file
    G_PRINT_RAW_TM = args.rawDataPrint
    G_COM_PORT = args.com_port
    G_TM_TIMEOUT = args.tm_timeout

