#!/usr/bin/python3.8
"""
@file
    OBSW_Config.py
@date
    01.11.2019
@brief
    Used to send single tcs and listen for replies after that
"""
import logging

from sendreceive.obsw_command_sender_receiver import CommandSenderReceiver
from sendreceive.obsw_tm_listener import TmListenerT
from comIF.obsw_com_interface import CommunicationInterface
from utility.obsw_tmtc_printer import TmTcPrinterT
import config.OBSW_Config as g


class SingleCommandSenderReceiver(CommandSenderReceiver):
    """
    Specific implementation of CommandSenderReceiver to send a single telecommand
    This object can be used by instantiating it and calling sendSingleTcAndReceiveTm()
    """
    def __init__(self, comInterface: CommunicationInterface, tmtcPrinter: TmTcPrinterT,
                 tmListener: TmListenerT, pusPacketTuple: tuple):
        """
        :param comInterface: CommunicationInterface object, passed on to CommandSenderReceiver
        :param tmListener: TmListener object which runs in the background and receives all TM
        :param tmtcPrinter: TmTcPrinter object, passed on to CommandSenderReceiver
        """
        super().__init__(comInterface=comInterface, tmListener=tmListener, tmtcPrinter=tmtcPrinter)
        self.faulty_input = False
        try:
            self.pus_packet, self.pus_packet_info = pusPacketTuple
        except TypeError:
            print("Invalid queue entry detected")
            logging.exception("Error")
            self.faulty_input = True

    def send_single_tc_and_receive_tm(self):
        """
        Send a single telecommand passed to the class and wait for replies
        :return:
        """
        if not self.faulty_input:
            self._tm_listener.modeId = g.ModeList.SingleCommandMode
            self._tm_listener.modeChangeEvent.set()
            self._com_interface.send_telecommand(self.pus_packet, self.pus_packet_info)
            while not self._reply_received:
                # wait until reply is received
                super()._check_for_first_reply()
            if self._reply_received:
                print("Single Command SenderReceiver: Reply received")
                self._tm_listener.modeOpFinished.set()
                print("Listening for packages ...")
