# -*- coding: utf-8 -*-
"""
Program: obsw_module_test.py
Date: 01.11.2019
Description: PUS Custom Service 8: Device Access, Native low-level commanding

@author: R. Mueller
"""
import struct

from tc.obsw_pus_tc_base import PusTelecommand, Deque
from tc.OBSW_TcService200 import packModeData


def packService2TestInto(tcQueue: Deque, calledExternally: bool = False) -> Deque:
    if calledExternally is False:
        tcQueue.appendleft(("print", "Testing Service 2"))
    objectId = bytearray([0x44, 0x00, 0xAF, 0xFE])  # dummy device
    # don't forget to set object mode raw (G_SERVICE 200) before calling this !
    # Set Raw Mode
    tcQueue.appendleft(("print", "\r\nTesting Service 2: Setting Raw Mode"))
    modeData = packModeData(objectId, 3, 0)
    command = PusTelecommand(service=200, subservice=1, SSC=2020, data=modeData)
    tcQueue.appendleft(command.packCommandTuple())
    # toggle wiretapping raw
    tcQueue.appendleft(("print", "\r\nTesting Service 2: Toggling Wiretapping Raw"))
    wiretappingToggleData = packWiretappingMode(objectId, 1)
    toggleWiretappingOnCommand = PusTelecommand(service=2, subservice=129, SSC=200,
                                                data=wiretappingToggleData)
    tcQueue.appendleft(toggleWiretappingOnCommand.packCommandTuple())
    # send raw command, _tm_data should be returned via TM[2,130] and TC[2,131]
    tcQueue.appendleft(("print", "\r\nTesting Service 2: Sending Raw Command"))
    rawCommand = struct.pack(">I", 666)
    rawData = objectId + rawCommand
    rawCommand = PusTelecommand(service=2, subservice=128, SSC=201, data=rawData)
    tcQueue.appendleft(rawCommand.packCommandTuple())
    # toggle wiretapping off
    tcQueue.appendleft(("print", "\r\nTesting Service 2: Toggle Wiretapping Off"))
    wiretappingToggleData = packWiretappingMode(objectId, 0)
    toggleWiretappingOffCommand = PusTelecommand(service=2, subservice=129, SSC=204,
                                                 data=wiretappingToggleData)
    tcQueue.appendleft(toggleWiretappingOffCommand.packCommandTuple())
    # send raw command which should be returned via TM[2,130]
    tcQueue.appendleft(("print", "\r\nTesting Service 2: Send second raw command"))
    command = PusTelecommand(service=2, subservice=128, SSC=205, data=rawData)
    tcQueue.appendleft(command.packCommandTuple())
    tcQueue.appendleft(("print", "\r"))
    if calledExternally is False:
        tcQueue.appendleft(("export", "log/tmtc_log_service2.txt"))
    return tcQueue


# wiretappingMode = 0: MODE_OFF, wiretappingMode = 1: MODE_RAW
def packWiretappingMode(objectId, wiretappingMode_):
    wiretappingMode = struct.pack(">B", wiretappingMode_)  # MODE_OFF : 0x00, MODE_RAW: 0x01
    wiretappingToggleData = objectId + wiretappingMode
    return wiretappingToggleData


def packSpecificService2TestInto(tcQueue, objectIdList, dataList, sscList, printStringList,
                                 calledExternally=False):
    pass
