# -*- coding: utf-8 -*-
"""
Program: obsw_module_test.py
Date: 01.11.2019
Description: PUS Custom Service 8: Device Access, Native low-level commanding

@author: R. Mueller
"""
import struct
from typing import Deque
from tc.obsw_pus_tc_base import PusTelecommand


def packService3TestInto(tcQueue: Deque) -> Deque:
    tcQueue.appendleft(("print", "Testing Service 3"))
    # adding custom defintion to hk using test pool variables
    sid1 = bytearray([0x00, 0x00, 0x43, 0x00])
    sid2 = bytearray([0x00, 0x00, 0x44, 0x00])
    sidGps = bytearray([0x00, 0x00, 0x1f, 0x00])
    collectionInterval = struct.pack('>f', 3)
    numberOfParameters = struct.pack('B', 5)
    p1 = bytearray([0x01, 0x01, 0x01, 0x01])
    p2 = bytearray([0x02, 0x02, 0x02, 0x02])
    p3 = bytearray([0x03, 0x03, 0x03, 0x03])
    p4 = bytearray([0x04, 0x04, 0x04, 0x04])
    p5 = bytearray([0x05, 0x05, 0x05, 0x05])
    hkDefinition1 = sid1 + collectionInterval + numberOfParameters + p1 + p2 + p3 + p4 + p5
    collectionInterval = struct.pack('>f', 6)
    hkDefinition2 = sid2 + collectionInterval + numberOfParameters + p1 + p2 + p3 + p4 + p5

    # deleting pre-defined test entry
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Deleting pre-defined HK definition"))
    command = PusTelecommand(service=3, subservice=3, SSC=3000, data=sid1)
    tcQueue.appendleft(command.packCommandTuple())

    # adding pre-defined definition to hk using test pool variables
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Adding pre-defined HK definition"))
    command = PusTelecommand(service=3, subservice=1, SSC=3010, data=hkDefinition1)
    tcQueue.appendleft(command.packCommandTuple())

    # adding custom definition to diagnostics using test pool variables
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Adding custom diganostics definition"))
    command = PusTelecommand(service=3, subservice=2, SSC=3020, data=hkDefinition2)
    tcQueue.appendleft(command.packCommandTuple())

    # enable custom hk definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Enable custom definition"))
    command = PusTelecommand(service=3, subservice=5, SSC=3030, data=sid1)
    tcQueue.appendleft(command.packCommandTuple())
    # enable custom diag definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Enable custom diagnostics definition"))
    command = PusTelecommand(service=3, subservice=7, SSC=3040, data=sid2)
    tcQueue.appendleft(command.packCommandTuple())
    # enable gps0
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Enable GPS definition"))
    command = PusTelecommand(service=3, subservice=5, SSC=3050, data=sidGps)
    tcQueue.appendleft(command.packCommandTuple())

    # maybe wait a bit to receive at least 2 packets..
    tcQueue.appendleft(("wait", 3))

    # Disable custom hk definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Disable custom definition"))
    command = PusTelecommand(service=3, subservice=6, SSC=3060, data=sid1)
    tcQueue.appendleft(command.packCommandTuple())
    # Disable custom diag definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Disable custom diagnostics definition"))
    command = PusTelecommand(service=3, subservice=8, SSC=3070, data=sid2)
    tcQueue.appendleft(command.packCommandTuple())
    # disable gps0
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Disable GPS definition"))
    command = PusTelecommand(service=3, subservice=6, SSC=3080, data=sidGps)
    tcQueue.appendleft(command.packCommandTuple())
    # report custom Diag definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Reporting diag definition"))
    command = PusTelecommand(service=3, subservice=11, SSC=3100, data=sid2)
    tcQueue.appendleft(command.packCommandTuple())
    # report gps definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Reporting GPS definition"))
    command = PusTelecommand(service=3, subservice=9, SSC=3110, data=sidGps)
    tcQueue.appendleft(command.packCommandTuple())
    # generate one custom hk definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Generate one custom hk definition"))
    command = PusTelecommand(service=3, subservice=27, SSC=3120, data=sid1)
    tcQueue.appendleft(command.packCommandTuple())
    # generate one custom diag definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Generate one custom diagnostics definition"))
    command = PusTelecommand(service=3, subservice=28, SSC=3120, data=sid2)
    tcQueue.appendleft(command.packCommandTuple())
    # generate one gps 0 definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Generate one gps 0 defintion"))
    command = PusTelecommand(service=3, subservice=27, SSC=3120, data=sidGps)
    tcQueue.appendleft(command.packCommandTuple())
    # modify custom hk definition interval
    newInterval = struct.pack('>f', 10.0)
    newIntervalCommand = sid1 + newInterval
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Changing pre-defined HK definition interval"))
    command = PusTelecommand(service=3, subservice=31, SSC=3090, data=newIntervalCommand)
    tcQueue.appendleft(command.packCommandTuple())
    # report custom HK definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Reporting pre-defined HK definition with changed interval"))
    command = PusTelecommand(service=3, subservice=9, SSC=3090, data=sid1)
    tcQueue.appendleft(command.packCommandTuple())
    # modify custom diag definition interval
    newIntervalCommand = sid2 + newInterval
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Changing custom diag HK definition interval"))
    command = PusTelecommand(service=3, subservice=32, SSC=3090, data=newIntervalCommand)
    tcQueue.appendleft(command.packCommandTuple())
    # report custom diag definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Reporting diag definition"))
    command = PusTelecommand(service=3, subservice=11, SSC=3100, data=sid2)
    tcQueue.appendleft(command.packCommandTuple())
    # append parameter to custom hk definiton
    # append parameter to custom diag definition

    # delete custom diag definition
    tcQueue.appendleft(("print", "\r\nTesting Service 3: Deleting custom diagnostics definition"))
    command = PusTelecommand(service=3, subservice=4, SSC=3120, data=sid2)
    tcQueue.appendleft(command.packCommandTuple())

    # do some basic testing on predefined structs too
    # e.g. add one variable, change interval, report them....
    tcQueue.appendleft(("export", "log/tmtc_log_service3.txt"))
    return tcQueue
