# -*- coding: utf-8 -*-
"""
Program: obsw_module_test.py
Date: 01.11.2019
Description: PUS Custom Service 8: Function Management, High-Level Commanding

@author: R. Mueller
"""
import struct
from typing import Deque

from tc.obsw_pus_tc_base import PusTelecommand
from tc.OBSW_TcService200 import packModeData


def packService8TestInto(tcQueue: Deque, calledExternally: bool = False) -> Deque:
    if calledExternally is False:
        tcQueue.appendleft(("print", "Testing Service 8"))
    objectId = bytearray([0x44, 0x00, 0xAF, 0xFE])
    # set mode on
    tcQueue.appendleft(("print", "\r\nTesting Service 8: Set Normal Mode"))
    modeData = packModeData(objectId, 1, 0)
    command = PusTelecommand(service=200, subservice=1, SSC=800, data=modeData)
    tcQueue.appendleft(command.packCommandTuple())
    # set mode normal
    tcQueue.appendleft(("print", "\r\nTesting Service 8: Set Normal Mode"))
    modeData = packModeData(objectId, 2, 0)
    command = PusTelecommand(service=200, subservice=1, SSC=810, data=modeData)
    tcQueue.appendleft(command.packCommandTuple())
    # Direct command which triggers completion reply
    tcQueue.appendleft(("print", "\r\nTesting Service 8: Trigger Completion Reply"))
    actionId = struct.pack(">I", 666)
    directCommand = objectId + actionId
    command = PusTelecommand(service=8, subservice=128, SSC=820, data=directCommand)
    tcQueue.appendleft(command.packCommandTuple())
    # Direct command which triggers _tm_data reply
    tcQueue.appendleft(("print", "\r\nTesting Service 8: Trigger Data Reply"))
    actionId = bytearray([0xC0, 0xC0, 0xBA, 0xBE])
    commandParam1 = bytearray([0xBA, 0xB0])
    commandParam2 = bytearray([0x00, 0x00, 0x00, 0x52, 0x4F, 0x42, 0x49, 0x4E])
    directCommand = objectId + actionId + commandParam1 + commandParam2
    command = PusTelecommand(service=8, subservice=128, SSC=830, data=directCommand)
    tcQueue.appendleft(command.packCommandTuple())
    # Direct command which triggers an additional step reply and one completion reply
    tcQueue.appendleft(("print", "\r\nTesting Service 8: Trigger Step and Completion Reply"))
    actionId = bytearray([0xBA, 0xDE, 0xAF, 0xFE])
    directCommand = objectId + actionId
    command = PusTelecommand(service=8, subservice=128, SSC=840, data=directCommand)
    tcQueue.appendleft(command.packCommandTuple())
    tcQueue.appendleft(("wait", 2))
    tcQueue.appendleft(("print", "\r"))
    if calledExternally is False:
        tcQueue.appendleft(("export", "log/tmtc_log_service8.txt"))
    return tcQueue


def packSpecificService8TestInto(tcQueue, objectIdList, actionIdList, dataList,
                                 sscList, printStringList, calledExternally=False):
    pass
