# -*- coding: utf-8 -*-
"""
Program: obsw_tm_service_1.py
Date: 30.12.2019
Description: Deserialize Pus Verification TM
Author: R. Mueller
"""
from tm.obsw_pus_tm_base import PusTelemetry
from typing import Dict
import struct

pusPacketInfoService1T = Dict[str, any]


class Service1TM(PusTelemetry):
    def __init__(self, byteArray):
        super().__init__(byteArray)
        self.tcErrorCode = False
        self.isStep = False
        # Failure Reports with error code
        self.tcPacketId = self._tm_data[0] << 8 | self._tm_data[1]
        self.tcSSC = ((self._tm_data[2] & 0x3F) << 8) | self._tm_data[3]
        self.specify_packet_info("Success Verification")
        if self.get_subservice() % 2 == 0:
            self.specify_packet_info("Failure Verficiation")
            self.tcErrorCode = True
            if self._data_field_header.subtype == 6:
                self.isStep = True
                self.append_packet_info(" : Step Failure")
                self.stepNumber = struct.unpack('>B', self._tm_data[4:5])[0]
                self.ErrCode = struct.unpack('>H', self._tm_data[5:7])[0]
                self.errorParam1 = struct.unpack('>I', self._tm_data[7:11])[0]
                self.errorParam2 = struct.unpack('>I', self._tm_data[11:15])[0]
            else:
                self.print_data()
                self.ErrCode = struct.unpack('>H', self._tm_data[4:6])[0]
                self.errorParam1 = struct.unpack('>I', self._tm_data[6:10])[0]
                self.errorParam2 = struct.unpack('>I', self._tm_data[10:14])[0]

        elif self.get_subservice() == 5:
            self.isStep = True
            self.append_packet_info(" : Step Success")
            self.stepNumber = struct.unpack('>B', self._tm_data[4:5])[0]

    def append_telemetry_content(self, array):
        super().append_telemetry_content(array)
        array.append(str(hex(self.tcPacketId)))
        array.append(str(self.tcSSC))
        if self.tcErrorCode:
            if self.isStep:
                array.append(str(self.stepNumber))
            array.append(str(hex(self.ErrCode)))
            array.append(str(hex(self.errorParam1)) + ", " + str(self.errorParam1))
            array.append(str(hex(self.errorParam2)) + ", " + str(self.errorParam2))
        elif self.isStep:
            array.append(str(self.stepNumber))

    def append_telemetry_column_headers(self, array):
        super().append_telemetry_column_headers(array)
        array.append("TC Packet ID")
        array.append("TC SSC")
        if self.tcErrorCode:
            if self.isStep:
                array.append("Step Number")
            array.append("Return Value")
            array.append("Error Param 1")
            array.append("Error Param 2")
        elif self.isStep:
            array.append("Step Number")

    def pack_tm_information(self) -> pusPacketInfoService1T:
        tmInformation = super().pack_tm_information()
        addInformation = {
            "tcPacketId": self.tcPacketId,
            "tcSSC": self.tcSSC,
        }
        tmInformation.update(addInformation)
        if self.tcErrorCode:
            tmInformation.update({"errCode": self.ErrCode})
        if self.isStep:
            tmInformation.update({"stepNumber": self.ErrCode})
        return tmInformation
